@@ -7,7 +7,7 @@ import sys
 import warnings
 from collections import OrderedDict
 from datetime import timedelta
-from itertools import chain
+from inspect import isasyncgen, isgenerator
 from logging import Logger
 from pathlib import Path
 from types import TracebackType
@@ -20,7 +20,6 @@ from typing import (
     cast,
     Coroutine,
     Dict,
-    IO,
     Iterable,
     List,
     NoReturn,
@@ -31,11 +30,15 @@ from typing import (
     Union,
     ValuesView,
 )
+from weakref import WeakSet
 
+from aiofiles import open as async_open
+from aiofiles.base import AiofilesContextManager
+from aiofiles.threadpool.binary import AsyncBufferedReader
 from hypercorn.asyncio import serve
 from hypercorn.config import Config as HyperConfig
 from hypercorn.typing import ASGIReceiveCallable, ASGISendCallable, Scope
-from werkzeug.datastructures import Headers
+from werkzeug.datastructures import Authorization, Headers
 from werkzeug.exceptions import HTTPException, InternalServerError
 from werkzeug.routing import MapAdapter, RoutingException
 from werkzeug.wrappers import Response as WerkzeugResponse
@@ -48,12 +51,13 @@ from .ctx import (
     _request_ctx_stack,
     _websocket_ctx_stack,
     AppContext,
+    copy_current_app_context,
     has_request_context,
     has_websocket_context,
     RequestContext,
     WebsocketContext,
 )
-from .globals import g, request, session, websocket
+from .globals import g, request, session
 from .helpers import (
     _split_blueprint_path,
     find_package,
@@ -66,10 +70,12 @@ from .json import JSONDecoder, JSONEncoder, jsonify, tojson_filter
 from .logging import create_logger, create_serving_logger
 from .routing import QuartMap, QuartRule
 from .scaffold import _endpoint_from_view_func, Scaffold
-from .sessions import SecureCookieSession, SecureCookieSessionInterface
+from .sessions import SecureCookieSessionInterface
 from .signals import (
     appcontext_tearing_down,
+    got_background_exception,
     got_request_exception,
+    got_serving_exception,
     got_websocket_exception,
     request_finished,
     request_started,
@@ -94,7 +100,7 @@ from .typing import (
     ASGIHTTPProtocol,
     ASGILifespanProtocol,
     ASGIWebsocketProtocol,
-    BeforeRequestCallable,
+    BeforeFirstRequestCallable,
     BeforeServingCallable,
     ErrorHandlerCallable,
     FilePath,
@@ -179,10 +185,7 @@ class Quart(Scaffold):
     config_class = Config
     env = ConfigAttribute("ENV")
     jinja_environment = Environment
-    jinja_options = {
-        "autoescape": True,
-        "extensions": ["jinja2.ext.autoescape", "jinja2.ext.with_"],
-    }
+    jinja_options: dict = {}
     json_decoder = JSONDecoder
     json_encoder = JSONEncoder
     lock_class = asyncio.Lock
@@ -256,7 +259,8 @@ class Quart(Scaffold):
         self.config = self.make_config(instance_relative_config)
 
         self.after_serving_funcs: List[Callable[[], Awaitable[None]]] = []
-        self.before_first_request_funcs: List[BeforeRequestCallable] = []
+        self.background_tasks: WeakSet[asyncio.Task] = WeakSet()
+        self.before_first_request_funcs: List[BeforeFirstRequestCallable] = []
         self.before_serving_funcs: List[Callable[[], Awaitable[None]]] = []
         self.blueprints: Dict[str, Blueprint] = OrderedDict()
         self.extensions: Dict[str, Any] = {}
@@ -368,17 +372,19 @@ class Quart(Scaffold):
             return package_path / "instance"
         return prefix / "var" / f"{self.name}-instance"
 
-    def open_instance_resource(self, path: FilePath, mode: str = "rb") -> IO[AnyStr]:
+    async def open_instance_resource(
+        self, path: FilePath, mode: str = "rb"
+    ) -> AiofilesContextManager[None, None, AsyncBufferedReader]:
         """Open a file for reading.
 
         Use as
 
         .. code-block:: python
 
-            with app.open_instance_resource(path) as file_:
-                file_.read()
+            async with await app.open_instance_resource(path) as file_:
+                await file_.read()
         """
-        return open(self.instance_path / file_path_to_path(path), mode)
+        return async_open(self.instance_path / file_path_to_path(path), mode)  # type: ignore
 
     @property
     def templates_auto_reload(self) -> bool:
@@ -438,16 +444,17 @@ class Quart(Scaffold):
         Arguments:
             context: The context to update (mutate).
         """
-        processors = self.template_context_processors[None]
+        names = [None]
         if has_request_context():
-            for blueprint in _request_ctx_stack.top.request.blueprints:
-                if blueprint in self.template_context_processors:
-                    processors = chain(  # type: ignore
-                        processors, self.template_context_processors[blueprint]
-                    )
+            names.extend(reversed(_request_ctx_stack.top.request.blueprints))
+        elif has_websocket_context():
+            names.extend(reversed(_websocket_ctx_stack.top.websocket.blueprints))
+
         extra_context: dict = {}
-        for processor in processors:
-            extra_context.update(await self.ensure_async(processor)())
+        for name in names:
+            for processor in self.template_context_processors[name]:
+                extra_context.update(await self.ensure_async(processor)())
+
         original = context.copy()
         context.update(extra_context)
         context.update(original)
@@ -487,10 +494,6 @@ class Quart(Scaffold):
     def register_blueprint(
         self,
         blueprint: Blueprint,
-        *,
-        url_prefix: Optional[str] = None,
-        url_defaults: Dict[str, Any] = None,
-        subdomain: Optional[str] = None,
         **options: Any,
     ) -> None:
         """Register a blueprint on the app.
@@ -504,13 +507,7 @@ class Quart(Scaffold):
             url_defaults: Blueprint routes will use these default values for view arguments.
             subdomain: Blueprint routes will match on this subdomain.
         """
-        blueprint.register(
-            app=self,
-            options={
-                **dict(url_prefix=url_prefix, url_defaults=url_defaults, subdomain=subdomain),
-                **options,
-            },
-        )
+        blueprint.register(self, options)
 
     def iter_blueprints(self) -> ValuesView[Blueprint]:
         """Return a iterator over the blueprints."""
@@ -739,8 +736,8 @@ class Quart(Scaffold):
 
     def before_first_request(
         self,
-        func: BeforeRequestCallable,
-    ) -> Callable[[], Awaitable[None]]:
+        func: BeforeFirstRequestCallable,
+    ) -> BeforeFirstRequestCallable:
         """Add a before **first** request function.
 
         This is designed to be used as a decorator, if used to
@@ -876,15 +873,13 @@ class Quart(Scaffold):
         This is used to assist when building urls, see
         :func:`~quart.helpers.url_for`.
         """
-        functions = self.url_default_functions[None]
+        names: List[Optional[str]] = [None]
         if "." in endpoint:
-            blueprints = _split_blueprint_path(endpoint.rsplit(".", 1)[0])
+            names.extend(reversed(_split_blueprint_path(endpoint.rsplit(".", 1)[0])))
 
-            for blueprint in blueprints:
-                functions = chain(functions, self.url_default_functions[blueprint])  # type: ignore
-
-        for function in functions:
-            function(endpoint, values)
+        for name in names:
+            for function in self.url_default_functions[name]:
+                function(endpoint, values)
 
     def handle_url_build_error(self, error: Exception, endpoint: str, values: dict) -> str:
         """Handle a build error.
@@ -901,14 +896,15 @@ class Quart(Scaffold):
     def _find_error_handler(self, error: Exception) -> Optional[ErrorHandlerCallable]:
         error_type, error_code = self._get_error_type_and_code(type(error))
 
-        blueprints = []
-        if _request_ctx_stack.top is not None:
-            blueprints = _request_ctx_stack.top.request.blueprints
-        elif _websocket_ctx_stack.top is not None:
-            blueprints = _websocket_ctx_stack.top.websocket.blueprints
+        names = []
+        if has_request_context():
+            names.extend(_request_ctx_stack.top.request.blueprints)
+        elif has_websocket_context():
+            names.extend(_websocket_ctx_stack.top.websocket.blueprints)
+        names.append(None)
 
         for code in [error_code, None]:
-            for name in chain(blueprints, [None]):
+            for name in names:
                 handlers = self.error_handler_spec[name].get(code)
 
                 if handlers is None:
@@ -1030,9 +1026,11 @@ class Quart(Scaffold):
             self.logger.error(
                 f"Exception on request {request_.method} {request_.path}", exc_info=exception_info
             )
-        if has_websocket_context():
+        elif has_websocket_context():
             websocket_ = _websocket_ctx_stack.top.websocket
             self.logger.error(f"Exception on websocket {websocket_.path}", exc_info=exception_info)
+        else:
+            self.logger.error("Exception", exc_info=exception_info)
 
     def raise_routing_exception(self, request: BaseRequestWebsocket) -> NoReturn:
         raise request.routing_exception
@@ -1098,12 +1096,11 @@ class Quart(Scaffold):
             request_context: The request context, optional as Flask
                 omits this argument.
         """
-        functions = self.teardown_request_funcs[None]
-        for blueprint in _request_ctx_stack.top.request.blueprints:
-            functions = chain(functions, self.teardown_request_funcs[blueprint])  # type: ignore
+        names = [*(request_context or _request_ctx_stack.top).request.blueprints, None]
+        for name in names:
+            for function in self.teardown_request_funcs[name]:
+                await self.ensure_async(function)(exc)
 
-        for function in functions:
-            await self.ensure_async(function)(exc)
         await request_tearing_down.send(self, exc=exc)
 
     async def do_teardown_websocket(
@@ -1117,12 +1114,11 @@ class Quart(Scaffold):
             websocket_context: The websocket context, optional as Flask
                 omits this argument.
         """
-        functions = self.teardown_websocket_funcs[None]
-        for blueprint in _websocket_ctx_stack.top.websocket.blueprints:
-            functions = chain(functions, self.teardown_websocket_funcs[blueprint])  # type: ignore
+        names = [*(websocket_context or _websocket_ctx_stack.top).websocket.blueprints, None]
+        for name in names:
+            for function in self.teardown_websocket_funcs[name]:
+                await self.ensure_async(function)(exc)
 
-        for function in functions:
-            await self.ensure_async(function)(exc)
         await websocket_tearing_down.send(self, exc=exc)
 
     async def do_teardown_appcontext(self, exc: Optional[BaseException]) -> None:
@@ -1339,6 +1335,7 @@ class Quart(Scaffold):
         root_path: str = "",
         http_version: str = "1.1",
         scope_base: Optional[dict] = None,
+        auth: Optional[Union[Authorization, Tuple[str, str]]] = None,
     ) -> RequestContext:
         """Create a request context for testing purposes.
 
@@ -1360,7 +1357,11 @@ class Quart(Scaffold):
             scheme: Scheme for the request, default http.
         """
         headers, path, query_string_bytes = make_test_headers_path_and_query_string(
-            self, path, headers, query_string
+            self,
+            path,
+            headers,
+            query_string,
+            auth,
         )
         request_body, body_headers = make_test_body_with_headers(data=data, form=form, json=json)
         headers.update(**body_headers)
@@ -1389,6 +1390,21 @@ class Quart(Scaffold):
         request.body.set_result(request_body)
         return self.request_context(request)
 
+    def add_background_task(self, func: Callable, *args: Any, **kwargs: Any) -> None:
+        async def _wrapper() -> None:
+            try:
+                await copy_current_app_context(func)(*args, **kwargs)
+            except Exception as error:
+                await self.handle_background_exception(error)
+
+        task = asyncio.get_event_loop().create_task(_wrapper())
+        self.background_tasks.add(task)
+
+    async def handle_background_exception(self, error: Exception) -> None:
+        await got_background_exception.send(self, exception=error)
+
+        self.log_exception(sys.exc_info())
+
     async def try_trigger_before_first_request_functions(self) -> None:
         """Trigger the before first request methods."""
         if self._got_first_request:
@@ -1446,10 +1462,16 @@ class Quart(Scaffold):
         if isinstance(value, HTTPException):
             response = value.get_response()  # type: ignore
         elif not isinstance(value, (Response, WerkzeugResponse)):
-            if isinstance(value, dict):
+            if (
+                isinstance(value, (str, bytes, bytearray))
+                or isgenerator(value)
+                or isasyncgen(value)
+            ):
+                response = self.response_class(value)
+            elif isinstance(value, dict):
                 response = jsonify(value)
             else:
-                response = self.response_class(value)
+                raise TypeError(f"The response value type ({type(value).__name__}) is not valid")
         else:
             response = value
 
@@ -1470,7 +1492,7 @@ class Quart(Scaffold):
             except Exception as error:
                 return await self.handle_exception(error)
             finally:
-                if request.scope.get("_quart._preserve_context", False):  # type: ignore
+                if request.scope.get("_quart._preserve_context", False):
                     self._preserved_context = request_context.copy()
 
     async def full_dispatch_request(
@@ -1501,19 +1523,18 @@ class Quart(Scaffold):
             request_context: The request context, optional as Flask
                 omits this argument.
         """
-        processors = self.url_value_preprocessors[None]
-        for blueprint in (request_context or _request_ctx_stack.top).request.blueprints:
-            processors = chain(processors, self.url_value_preprocessors[blueprint])  # type: ignore
-        for processor in processors:
-            processor(request.endpoint, request.view_args)
+        names = [None, *reversed((request_context or _request_ctx_stack.top).request.blueprints)]
+
+        for name in names:
+            for processor in self.url_value_preprocessors[name]:
+                processor(request.endpoint, request.view_args)
+
+        for name in names:
+            for function in self.before_request_funcs[name]:
+                result = await self.ensure_async(function)()
+                if result is not None:
+                    return result
 
-        functions = self.before_request_funcs[None]
-        for blueprint in _request_ctx_stack.top.request.blueprints:
-            functions = chain(functions, self.before_request_funcs[blueprint])  # type: ignore
-        for function in functions:
-            result = await self.ensure_async(function)()
-            if result is not None:
-                return result
         return None
 
     async def dispatch_request(
@@ -1570,14 +1591,15 @@ class Quart(Scaffold):
             request_context: The request context, optional as Flask
                 omits this argument.
         """
-        functions = (request_context or _request_ctx_stack.top)._after_request_functions
-        for blueprint in (request_context or _request_ctx_stack.top).request.blueprints:
-            functions = chain(functions, self.after_request_funcs[blueprint])
-        functions = chain(functions, self.after_request_funcs[None])
+        names = [*(request_context or _request_ctx_stack.top).request.blueprints, None]
 
-        for function in functions:
+        for function in (request_context or _request_ctx_stack.top)._after_request_functions:
             response = await self.ensure_async(function)(response)
 
+        for name in names:
+            for function in reversed(self.after_request_funcs[name]):
+                response = await self.ensure_async(function)(response)
+
         session_ = (request_context or _request_ctx_stack.top).session
         if not self.session_interface.is_null_session(session_):
             await self.ensure_async(self.session_interface.save_session)(self, session_, response)
@@ -1622,19 +1644,21 @@ class Quart(Scaffold):
             websocket_context: The websocket context, optional as Flask
                 omits this argument.
         """
-        processors = self.url_value_preprocessors[None]
-        for blueprint in _websocket_ctx_stack.top.websocket.blueprints:
-            processors = chain(processors, self.url_value_preprocessors[blueprint])  # type: ignore
-        for processor in processors:
-            processor(websocket.endpoint, websocket.view_args)
+        names = [
+            None,
+            *reversed((websocket_context or _websocket_ctx_stack.top).websocket.blueprints),
+        ]
+
+        for name in names:
+            for processor in self.url_value_preprocessors[name]:
+                processor(request.endpoint, request.view_args)
+
+        for name in names:
+            for function in self.before_request_funcs[name]:
+                result = await self.ensure_async(function)()
+                if result is not None:
+                    return result
 
-        functions = self.before_websocket_funcs[None]
-        for blueprint in _websocket_ctx_stack.top.websocket.blueprints:
-            functions = chain(functions, self.before_websocket_funcs[blueprint])  # type: ignore
-        for function in functions:
-            result = await self.ensure_async(function)()
-            if result is not None:
-                return result
         return None
 
     async def dispatch_websocket(
@@ -1691,23 +1715,18 @@ class Quart(Scaffold):
             websocket_context: The websocket context, optional as Flask
                 omits this argument.
         """
-        functions = (websocket_context or _websocket_ctx_stack.top)._after_websocket_functions
-        for blueprint in _websocket_ctx_stack.top.websocket.blueprints:
-            functions = chain(functions, self.after_websocket_funcs[blueprint])
-        functions = chain(functions, self.after_websocket_funcs[None])
+        names = [*(websocket_context or _websocket_ctx_stack.top).websocket.blueprints, None]
 
-        for function in functions:
+        for function in (websocket_context or _websocket_ctx_stack.top)._after_websocket_functions:
             response = await self.ensure_async(function)(response)
 
-        session_ = (websocket_context or _request_ctx_stack.top).session
+        for name in names:
+            for function in reversed(self.after_websocket_funcs[name]):
+                response = await self.ensure_async(function)(response)
+
+        session_ = (websocket_context or _websocket_ctx_stack.top).session
         if not self.session_interface.is_null_session(session_):
-            if response is None and isinstance(session_, SecureCookieSession) and session_.modified:
-                self.logger.exception(
-                    "Secure Cookie Session modified during websocket handling. "
-                    "These modifications will be lost as a cookie cannot be set."
-                )
-            else:
-                await self.session_interface.save_session(self, session_, response)
+            await self.session_interface.save_session(self, session_, response)
         return response
 
     async def __call__(
@@ -1750,23 +1769,33 @@ class Quart(Scaffold):
     async def startup(self) -> None:
         self._got_first_request = False
 
-        async with self.app_context():
-            for func in self.before_serving_funcs:
-                await self.ensure_async(func)()
-            for gen in self.while_serving_gens:
-                await gen.__anext__()
+        try:
+            async with self.app_context():
+                for func in self.before_serving_funcs:
+                    await self.ensure_async(func)()
+                for gen in self.while_serving_gens:
+                    await gen.__anext__()
+        except Exception as error:
+            await got_serving_exception.send(self, exception=error)
+            self.log_exception(sys.exc_info())
 
     async def shutdown(self) -> None:
-        async with self.app_context():
-            for func in self.after_serving_funcs:
-                await self.ensure_async(func)()
-            for gen in self.while_serving_gens:
-                try:
-                    await gen.__anext__()
-                except StopAsyncIteration:
-                    pass
-                else:
-                    raise RuntimeError("While serving generator didn't terminate")
+        await asyncio.gather(*self.background_tasks)
+
+        try:
+            async with self.app_context():
+                for func in self.after_serving_funcs:
+                    await self.ensure_async(func)()
+                for gen in self.while_serving_gens:
+                    try:
+                        await gen.__anext__()
+                    except StopAsyncIteration:
+                        pass
+                    else:
+                        raise RuntimeError("While serving generator didn't terminate")
+        except Exception as error:
+            await got_serving_exception.send(self, exception=error)
+            self.log_exception(sys.exc_info())
 
 
 def _cancel_all_tasks(loop: asyncio.AbstractEventLoop) -> None:
@@ -1776,7 +1805,7 @@ def _cancel_all_tasks(loop: asyncio.AbstractEventLoop) -> None:
 
     for task in tasks:
         task.cancel()
-    loop.run_until_complete(asyncio.gather(*tasks, loop=loop, return_exceptions=True))
+    loop.run_until_complete(asyncio.gather(*tasks, return_exceptions=True))
 
     for task in tasks:
         if not task.cancelled() and task.exception() is not None:
