@@ -284,8 +284,8 @@ class ASGIWebsocketConnection:
                 message["headers"] = encode_headers(headers)
             elif headers:
                 warnings.warn("The ASGI Server does not support accept headers, headers not sent")
-            await send(message)
             self._accepted = True
+            await send(message)
 
     async def close_connection(self, send: ASGISendCallable, code: int, reason: str) -> None:
         if self._closed:
@@ -337,7 +337,7 @@ class ASGILifespan:
                 break
 
 
-async def _cancel_tasks(tasks: Set[asyncio.Future]) -> None:
+async def _cancel_tasks(tasks: Set[asyncio.Task]) -> None:
     # Cancel any pending, and wait for the cancellation to
     # complete i.e. finish any remaining work.
     for task in tasks:
@@ -346,8 +346,8 @@ async def _cancel_tasks(tasks: Set[asyncio.Future]) -> None:
     _raise_exceptions(tasks)
 
 
-def _raise_exceptions(tasks: Set[asyncio.Future]) -> None:
-    # Raise any unexcepted exceptions
+def _raise_exceptions(tasks: Set[asyncio.Task]) -> None:
+    # Raise any unexpected exceptions
     for task in tasks:
         if not task.cancelled() and task.exception() is not None:
             raise task.exception()
