@@ -109,7 +109,7 @@ class Body:
         self._has_data.set()
 
     def set_result(self, data: bytes) -> None:
-        """Convienience method, mainly for testing."""
+        """Convenience method, mainly for testing."""
         self.append(data)
         self.set_complete()
 
@@ -135,6 +135,7 @@ class Request(BaseRequestWebsocket):
     body_class = Body
     form_data_parser_class = FormDataParser
     json_module = json
+    lock_class = asyncio.Lock
 
     def __init__(
         self,
@@ -180,6 +181,7 @@ class Request(BaseRequestWebsocket):
         self._cached_json: Dict[bool, Any] = {False: Ellipsis, True: Ellipsis}
         self._form: Optional[MultiDict] = None
         self._files: Optional[MultiDict] = None
+        self._parsing_lock = self.lock_class()
         self._send_push_promise = send_push_promise
 
     @property
@@ -222,15 +224,8 @@ class Request(BaseRequestWebsocket):
             await self._load_form_data()
 
         try:
-            body_future = asyncio.ensure_future(self.body)
-            raw_data = await asyncio.wait_for(body_future, timeout=self.body_timeout)
+            raw_data = await asyncio.wait_for(self.body, timeout=self.body_timeout)
         except asyncio.TimeoutError:
-            body_future.cancel()
-            try:
-                await body_future
-            except asyncio.CancelledError:
-                pass
-
             raise RequestTimeout()
         else:
             if not cache:
@@ -289,17 +284,21 @@ class Request(BaseRequestWebsocket):
         )
 
     async def _load_form_data(self) -> None:
-        if self._form is None:
-            parser = self.make_form_data_parser()
-            self._form, self._files = await asyncio.wait_for(
-                parser.parse(
-                    self.body,
-                    self.mimetype,
-                    self.content_length,
-                    self.mimetype_params,
-                ),
-                timeout=self.body_timeout,
-            )
+        async with self._parsing_lock:
+            if self._form is None:
+                parser = self.make_form_data_parser()
+                try:
+                    self._form, self._files = await asyncio.wait_for(
+                        parser.parse(
+                            self.body,
+                            self.mimetype,
+                            self.content_length,
+                            self.mimetype_params,
+                        ),
+                        timeout=self.body_timeout,
+                    )
+                except asyncio.TimeoutError:
+                    raise RequestTimeout()
 
     @property
     async def json(self) -> Any:
@@ -342,7 +341,7 @@ class Request(BaseRequestWebsocket):
         Arguments:
             error: The exception raised during parsing.
         Returns:
-            Any value returned (if overidden) will be used as the
+            Any value returned (if overridden) will be used as the
             default for any get_json calls.
         """
         if current_app and current_app.debug:
